@@ -22,6 +22,10 @@
 #include <config.h>
 #endif
 
+#ifdef GDK_MULTIHEAD_SAFE
+#undef GDK_MULTIHEAD_SAFE
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
@@ -44,6 +48,8 @@
 #include "mywindow.h"
 #include "frame.h"
 #include "client.h"
+#include "transients.h"
+#include "focus.h"
 #include "events.h"
 #include "menu.h"
 #include "hints.h"
@@ -114,13 +120,13 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
     button = ev->xbutton.button;
     x = xcurrent = ev->xbutton.x_root;
     y = ycurrent = ev->xbutton.y_root;
-    t0 = GDK_CURRENT_TIME;
+    t0 = CurrentTime;
     total = 0;
     clicks = 1;
 
     while ((ABS (x - xcurrent) < 2) && (ABS (y - ycurrent) < 2)
         && (total < params.dbl_click_time)
-        && ((GDK_CURRENT_TIME - t0) < params.dbl_click_time))
+        && ((CurrentTime - t0) < params.dbl_click_time))
     {
         g_usleep (10000);
         total += 10;
@@ -193,7 +199,7 @@ reset_timeout (void)
 static void
 moveRequest (Client * c, XEvent * ev)
 {
-    if (CLIENT_FLAG_TEST_AND_NOT (c, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
+    if (FLAG_TEST_AND_NOT (c->flags, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
     {
         clientMove (c, ev);
     }
@@ -202,14 +208,14 @@ moveRequest (Client * c, XEvent * ev)
 static void
 resizeRequest (Client * c, int corner, XEvent * ev)
 {
-    clientSetFocus (c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+    clientSetFocus (c, NO_FOCUS_FLAG);
 
-    if (CLIENT_FLAG_TEST_ALL (c,
+    if (FLAG_TEST_ALL (c->flags,
             CLIENT_FLAG_HAS_RESIZE | CLIENT_FLAG_IS_RESIZABLE))
     {
         clientResize (c, corner, ev);
     }
-    else if (CLIENT_FLAG_TEST_AND_NOT (c, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
+    else if (FLAG_TEST_AND_NOT (c->flags, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
     {
         clientMove (c, ev);
     }
@@ -270,7 +276,7 @@ handleMotionNotify (XMotionEvent * ev)
                 workspaceSwitch (workspace + 1, NULL);
             }
             while (XCheckWindowEvent(dpy, ev->window, PointerMotionMask, (XEvent *) ev))
-                ; /* void */
+                ; /* Skip event */
         }
     }
 }
@@ -309,7 +315,7 @@ handleKeyPress (XKeyEvent * ev)
             case KEY_RESIZE_DOWN:
             case KEY_RESIZE_LEFT:
             case KEY_RESIZE_RIGHT:
-                if (CLIENT_FLAG_TEST_ALL (c,
+                if (FLAG_TEST_ALL (c->flags,
                         CLIENT_FLAG_HAS_RESIZE | CLIENT_FLAG_IS_RESIZABLE))
                 {
                     clientResize (c, CORNER_BOTTOM_RIGHT, (XEvent *) ev);
@@ -322,7 +328,7 @@ handleKeyPress (XKeyEvent * ev)
                 clientClose (c);
                 break;
             case KEY_HIDE_WINDOW:
-                if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER) && CLIENT_CAN_HIDE_WINDOW (c))
+                if (CLIENT_CAN_HIDE_WINDOW (c))
                 {
                     clientHide (c, c->win_workspace, TRUE);
                 }
@@ -339,13 +345,10 @@ handleKeyPress (XKeyEvent * ev)
             case KEY_SHADE_WINDOW:
                 clientToggleShaded (c);
                 break;
-	    case KEY_STICK_WINDOW:
-		if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER) && CLIENT_CAN_STICK_WINDOW (c))
-		{
-		    clientToggleSticky (c, TRUE);
-		    frameDraw (c, FALSE, FALSE);
-		}
-		break;
+            case KEY_STICK_WINDOW:
+                clientToggleSticky (c, TRUE);
+                frameDraw (c, FALSE, FALSE);
+                break;
             case KEY_RAISE_WINDOW:
                 clientRaise (c);
                 clientPassGrabButton1 (NULL);
@@ -479,7 +482,7 @@ button1Action (Client * c, XButtonEvent * ev)
     g_return_if_fail (c != NULL);
     g_return_if_fail (ev != NULL);
 
-    clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
+    clientSetFocus (c, NO_FOCUS_FLAG);
     clientRaise (c);
     clientPassGrabButton1 (c);
 
@@ -545,7 +548,7 @@ titleButton (Client * c, int state, XButtonEvent * ev)
         }
         else
         {
-            clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
+            clientSetFocus (c, NO_FOCUS_FLAG);
             if (params.raise_on_click)
             {
                 clientRaise (c);
@@ -567,7 +570,7 @@ titleButton (Client * c, int state, XButtonEvent * ev)
     else if (ev->button == Button4)
     {
         /* Mouse wheel scroll up */
-        if (!CLIENT_FLAG_TEST (c, CLIENT_FLAG_SHADED))
+        if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             clientShade (c);
         }
@@ -575,7 +578,7 @@ titleButton (Client * c, int state, XButtonEvent * ev)
     else if (ev->button == Button5)
     {
         /* Mouse wheel scroll down */
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_SHADED))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             clientUnshade (c);
         }
@@ -654,7 +657,7 @@ handleButtonPress (XButtonEvent * ev)
         {
             if (ev->button <= Button3)
             {
-                clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
+                clientSetFocus (c, NO_FOCUS_FLAG);
                 if (params.raise_on_click)
                 {
                     clientRaise (c);
@@ -688,7 +691,7 @@ handleButtonPress (XButtonEvent * ev)
                 }
                 else
                 {
-                    clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
+                    clientSetFocus (c, NO_FOCUS_FLAG);
                     if (params.raise_on_click)
                     {
                         clientRaise (c);
@@ -758,8 +761,8 @@ handleButtonPress (XButtonEvent * ev)
                 {
                     clientPassGrabButton1 (c);
                 }
-                clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
-                if ((params.raise_on_click) || !CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER))
+                clientSetFocus (c, NO_FOCUS_FLAG);
+                if ((params.raise_on_click) || !FLAG_TEST (c->flags, CLIENT_FLAG_HAS_BORDER))
                 {
                     clientRaise (c);
                     clientPassGrabButton1 (c);
@@ -787,7 +790,7 @@ handleButtonPress (XButtonEvent * ev)
     }
     else
     {
-        XUngrabPointer (dpy, GDK_CURRENT_TIME);
+        XUngrabPointer (dpy, CurrentTime);
         XSendEvent (dpy, gnome_win, FALSE, SubstructureNotifyMask,
             (XEvent *) ev);
     }
@@ -843,13 +846,13 @@ handleMapRequest (XMapRequestEvent * ev)
     if (c)
     {
         TRACE ("handleMapRequest: clientShow");
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAP_PENDING))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_MAP_PENDING))
         {
             TRACE ("Ignoring MapRequest on window (0x%lx)", ev->window);
             return;
         }
         clientShow (c, TRUE);
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY) ||
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY) ||
             (c->win_workspace == workspace))
         {
             clientFocusNew(c);
@@ -874,9 +877,9 @@ handleMapNotify (XMapEvent * ev)
     if (c)
     {
         TRACE ("MapNotify for \"%s\" (0x%lx)", c->name, c->window);
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAP_PENDING))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_MAP_PENDING))
         {
-            CLIENT_FLAG_UNSET (c, CLIENT_FLAG_MAP_PENDING);
+            FLAG_UNSET (c->flags, CLIENT_FLAG_MAP_PENDING);
         }
     }
 }
@@ -907,7 +910,7 @@ handleUnmapNotify (XUnmapEvent * ev)
         TRACE ("UnmapNotify for \"%s\" (0x%lx)", c->name, c->window);
         TRACE ("ignore_unmaps for \"%s\" is %i", c->name, c->ignore_unmap);
 
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAP_PENDING))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_MAP_PENDING))
         {
             /* 
              * This UnmapNotify event is caused by reparenting
@@ -1035,7 +1038,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
 
         TRACE ("handleConfigureRequest managed window \"%s\" (0x%lx)",
             c->name, c->window);
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MOVING_RESIZING))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_MOVING_RESIZING))
         {
             /* Sorry, but it's not the right time for configure request */
             return;
@@ -1046,7 +1049,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
             ev->value_mask &= ~(CWSibling | CWStackMode);
         }
         clientCoordGravitate (c, APPLY, &wc.x, &wc.y);
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_FULLSCREEN))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
         {
             int cx, cy;
 
@@ -1082,7 +1085,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
         /* Still a move/resize after cleanup? */
         if (ev->value_mask & (CWX | CWY | CWWidth | CWHeight))
         {
-            if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAXIMIZED))
+            if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
                 clientRemoveMaximizeFlag (c);
             }
@@ -1097,9 +1100,9 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
         if ((ev->value_mask & CWStackMode) && (wc.stack_mode == Above))
         {
             if ((c->win_workspace == workspace) || 
-                (CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY)))
+                (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY)))
             {
-                if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_HIDDEN))
+                if (FLAG_TEST (c->flags, CLIENT_FLAG_HIDDEN))
                 {
                     clientShow (c, TRUE);
                 }
@@ -1138,7 +1141,7 @@ handleEnterNotify (XCrossingEvent * ev)
         TRACE ("EnterNotify window is \"%s\"", c->name);
         if (!(c->type & (WINDOW_DOCK | WINDOW_DESKTOP)))
         {
-            clientSetFocus (c, ev->time, FOCUS_FORCE);
+            clientSetFocus (c, FOCUS_FORCE);
             if (!(params.raise_on_click))
             {
                 clientPassGrabButton1 (c);
@@ -1205,11 +1208,10 @@ handleFocusIn (XFocusChangeEvent * ev)
         /* Handle focus transition to root (means that an unknown
            window has vanished and the focus is returned to the root
          */
-         
         c = clientGetFocus ();
         if (c)
         {
-            clientSetFocus (c, GDK_CURRENT_TIME, FOCUS_FORCE);
+            clientSetFocus (c, FOCUS_FORCE);
         }
         return;
     }
@@ -1287,6 +1289,8 @@ static void
 handlePropertyNotify (XPropertyEvent * ev)
 {
     Client *c = NULL;
+    char *names;
+    int length;
 
     TRACE ("entering handlePropertyNotify");
 
@@ -1309,7 +1313,7 @@ handlePropertyNotify (XPropertyEvent * ev)
                 free (c->name);
             }
             getWindowName (dpy, c->window, &c->name);
-            CLIENT_FLAG_SET (c, CLIENT_FLAG_NAME_CHANGED);
+            FLAG_SET (c->flags, CLIENT_FLAG_NAME_CHANGED);
             frameDraw (c, TRUE, FALSE);
         }
         else if (ev->atom == motif_wm_hints)
@@ -1353,7 +1357,7 @@ handlePropertyNotify (XPropertyEvent * ev)
             clientGetNetWmType (c);
             frameDraw (c, TRUE, FALSE);
         }
-        else if (ev->atom == net_wm_strut)
+        else if ((ev->atom == net_wm_strut) || (ev->atom == net_wm_strut_partial))
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_strut notify",
                 c->name, c->window);
@@ -1367,6 +1371,13 @@ handlePropertyNotify (XPropertyEvent * ev)
                 clientInstallColormaps (c);
             }
         }
+        else if (ev->atom == net_wm_user_time)
+        {
+            if (getNetWMUserTime (dpy, c->window, &c->user_time))
+	    {
+                FLAG_SET (c->flags, CLIENT_FLAG_HAS_USER_TIME);
+	    }
+        }
 #ifdef HAVE_STARTUP_NOTIFICATION
         else if (ev->atom == net_startup_id)
         {
@@ -1380,6 +1391,14 @@ handlePropertyNotify (XPropertyEvent * ev)
 #endif
 
     }
+    else if (ev->atom == net_desktop_names)
+    {
+        TRACE ("root has received a net_desktop_names notify");
+        if (getUTF8String (dpy, root, net_desktop_names, &names, &length))
+        {
+            workspaceSetNames (names, length);
+        }
+    }
     else if (ev->atom == gnome_panel_desktop_area)
     {
         TRACE ("root has received a gnome_panel_desktop_area notify");
@@ -1408,7 +1427,7 @@ handleClientMessage (XClientMessageEvent * ev)
             TRACE
                 ("client \"%s\" (0x%lx) has received a wm_change_state event",
                 c->name, c->window);
-            if (!CLIENT_FLAG_TEST (c, CLIENT_FLAG_HIDDEN) && 
+            if (!FLAG_TEST (c->flags, CLIENT_FLAG_HIDDEN) && 
                  CLIENT_CAN_HIDE_WINDOW (c))
             {
                 clientHide (c, c->win_workspace, TRUE);
@@ -1448,7 +1467,7 @@ handleClientMessage (XClientMessageEvent * ev)
             {
                 if (ev->data.l[0] == ALL_WORKSPACES)
                 {
-                    if (CLIENT_FLAG_TEST_AND_NOT (c, CLIENT_FLAG_HAS_STICK,
+                    if (FLAG_TEST_AND_NOT (c->flags, CLIENT_FLAG_HAS_STICK,
                             CLIENT_FLAG_STICKY))
                     {
                         clientStick (c, TRUE);
@@ -1457,7 +1476,7 @@ handleClientMessage (XClientMessageEvent * ev)
                 }
                 else
                 {
-                    if (CLIENT_FLAG_TEST_ALL (c,
+                    if (FLAG_TEST_ALL (c->flags,
                             CLIENT_FLAG_HAS_STICK | CLIENT_FLAG_STICKY))
                     {
                         clientUnstick (c, TRUE);
@@ -1502,7 +1521,7 @@ handleClientMessage (XClientMessageEvent * ev)
             clientSetWorkspace (c, workspace, TRUE);
             clientShow (c, TRUE);
             clientRaise (c);
-            clientSetFocus (c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+            clientSetFocus (c, NO_FOCUS_FLAG);
             clientPassGrabButton1 (c);
         }
     }
@@ -1536,6 +1555,13 @@ handleClientMessage (XClientMessageEvent * ev)
             TRACE ("root has received a net_system_tray_manager event");
             systray = getSystrayWindow (dpy);
         }
+        else if ((ev->message_type == net_showing_desktop)
+            && (ev->format == 32))
+        {
+            TRACE ("root has received a net_showing_desktop event");
+	    clientToggleShowDesktop (ev->data.l[0]);
+	    setHint (dpy, root, net_showing_desktop, ev->data.l[0]);
+        }
         else
         {
             TRACE ("unidentified client message for window 0x%lx",
@@ -1734,6 +1760,14 @@ menu_callback (Menu * menu, MenuOp op, Window client_xwindow,
             frameDraw (c, FALSE, FALSE);
             clientClose (c);
             break;
+        case MENU_OP_CONTEXT_HELP:
+            clientEnterContextMenuState (c);
+            frameDraw (c, FALSE, FALSE);
+            break;
+        case MENU_OP_ABOVE:
+        case MENU_OP_NORMAL:
+            clientToggleAbove (c);
+            /* Fall thru */
         default:
             frameDraw (c, FALSE, FALSE);
             break;
@@ -1761,12 +1795,12 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         ops = MENU_OP_DELETE | MENU_OP_MINIMIZE_ALL | MENU_OP_WORKSPACES;
         insensitive = 0;
 
-        if (!CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_CLOSE))
+        if (!FLAG_TEST (c->flags, CLIENT_FLAG_HAS_CLOSE))
         {
             insensitive |= MENU_OP_DELETE;
         }
 
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAXIMIZED))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
         {
             ops |= MENU_OP_UNMAXIMIZE;
             if (!CLIENT_CAN_MAXIMIZE_WINDOW (c))
@@ -1783,7 +1817,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
             }
         }
 
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_HIDDEN))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_HIDDEN))
         {
             ops |= MENU_OP_UNMINIMIZE;
             if (!CLIENT_CAN_HIDE_WINDOW (c))
@@ -1800,7 +1834,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
             }
         }
 
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_SHADED))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             ops |= MENU_OP_UNSHADE;
         }
@@ -1809,10 +1843,10 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
             ops |= MENU_OP_SHADE;
         }
 
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
             ops |= MENU_OP_UNSTICK;
-            if (!CLIENT_CAN_STICK_WINDOW(c))
+            if (!FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STICK))
             {
                 insensitive |= MENU_OP_UNSTICK;
             }
@@ -1820,15 +1854,41 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         else
         {
             ops |= MENU_OP_STICK;
-            if (!CLIENT_CAN_STICK_WINDOW(c))
+            if (!FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STICK))
             {
                 insensitive |= MENU_OP_STICK;
             }
         }
+    
+        /* KDE extension */
+        clientGetWMProtocols(c);
+        if (FLAG_TEST (c->wm_flags, WM_FLAG_CONTEXT_HELP))
+        {
+            ops |= MENU_OP_CONTEXT_HELP;
+        }
+
+        if (FLAG_TEST(c->flags, CLIENT_FLAG_ABOVE))
+        {
+            ops |= MENU_OP_NORMAL;
+            if (clientIsTransientOrModal (c) || 
+                FLAG_TEST (c->flags, CLIENT_FLAG_BELOW | CLIENT_FLAG_FULLSCREEN))
+            {
+                insensitive |= MENU_OP_NORMAL;
+            }
+        }
+        else
+        {
+            ops |= MENU_OP_ABOVE;
+            if (clientIsTransientOrModal (c) || 
+                FLAG_TEST (c->flags, CLIENT_FLAG_BELOW | CLIENT_FLAG_FULLSCREEN))
+            {
+                insensitive |= MENU_OP_ABOVE;
+            }
+        }
 
         if (clientIsTransientOrModal (c)
-            || !CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_STICK)
-            || CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY))
+            || !FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STICK)
+            || FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
             insensitive |= MENU_OP_WORKSPACES;
         }
@@ -1868,9 +1928,11 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
                                         MyDisplayFullWidth (dpy, screen),
                                         MyDisplayFullHeight (dpy, screen), 
                                         NoEventMask);
-    menu =
-        menu_default (ops, insensitive, menu_callback, c->win_workspace,
-        params.workspace_count, c);
+
+    menu = menu_default (ops, insensitive, menu_callback, c->win_workspace,
+                         params.workspace_count, params.workspace_names,
+                         params.workspace_names_length, c);
+
     if (!menu_popup (menu, x, y, ev->button, ev->time))
     {
         TRACE ("Cannot open menu");
