@@ -77,7 +77,9 @@ Atom net_close_window;
 Atom net_current_desktop;
 Atom net_desktop_geometry;
 Atom net_desktop_viewport;
+Atom net_desktop_names;
 Atom net_number_of_desktops;
+Atom net_showing_desktop;
 Atom net_startup_id;
 Atom net_supported;
 Atom net_supporting_wm_check;
@@ -109,6 +111,7 @@ Atom net_wm_state_skip_pager;
 Atom net_wm_state_skip_taskbar;
 Atom net_wm_state_sticky;
 Atom net_wm_strut;
+Atom net_wm_strut_partial;
 Atom net_wm_window_type;
 Atom net_wm_window_type_desktop;
 Atom net_wm_window_type_dialog;
@@ -118,11 +121,15 @@ Atom net_wm_window_type_normal;
 Atom net_wm_window_type_splashscreen;
 Atom net_wm_window_type_toolbar;
 Atom net_wm_window_type_utility;
+Atom net_wm_user_time;
 Atom net_workarea;
 Atom utf8_string;
 
-/* Systray similation for older KDE apps */
+/* KDE extension */
+Atom kde_net_wm_context_help;
 Atom kde_net_wm_system_tray_window_for;
+
+/* Systray similation for older KDE apps */
 Atom net_system_tray_manager;
 Atom net_system_tray_selection;
 Atom net_system_tray_opcode;
@@ -248,6 +255,11 @@ getWMProtocols (Display * dpy, Window w)
             {
                 result |= WM_PROTOCOLS_DELETE_WINDOW;
             }
+            /* KDE extension */
+            if (*ap == (Atom) kde_net_wm_context_help)
+            {
+                result |= WM_PROTOCOLS_CONTEXT_HELP;
+            }
         }
     }
     else
@@ -266,6 +278,11 @@ getWMProtocols (Display * dpy, Window w)
                 {
                     result |= WM_PROTOCOLS_DELETE_WINDOW;
                 }
+                /* KDE extension */
+                if (*ap == (Atom) kde_net_wm_context_help)
+                {
+                    result |= WM_PROTOCOLS_CONTEXT_HELP;
+                }
             }
         }
     }
@@ -297,6 +314,17 @@ initGnomeHints (Display * dpy)
     win_workspace = XInternAtom (dpy, "_WIN_WORKSPACE", FALSE);
 }
 
+void
+initKDEHints (Display * dpy)
+{
+    TRACE ("entering initKDEHints");
+
+    kde_net_wm_context_help = 
+        XInternAtom (dpy, "_NET_WM_CONTEXT_HELP", FALSE);
+    kde_net_wm_system_tray_window_for = 
+        XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
+}
+
 void
 initSystrayHints (Display * dpy, int nscreen)
 {
@@ -308,8 +336,6 @@ initSystrayHints (Display * dpy, int nscreen)
     net_system_tray_manager   = XInternAtom (dpy, "MANAGER", FALSE);
     net_system_tray_opcode    = XInternAtom (dpy, "_NET_SYSTEM_TRAY_OPCODE", FALSE);
     net_system_tray_selection = XInternAtom (dpy, selection, FALSE);
-    kde_net_wm_system_tray_window_for = 
-        XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
 }
 
 gboolean
@@ -401,8 +427,10 @@ initNetHints (Display * dpy)
     net_current_desktop = XInternAtom (dpy, "_NET_CURRENT_DESKTOP", FALSE);
     net_desktop_geometry = XInternAtom (dpy, "_NET_DESKTOP_GEOMETRY", FALSE);
     net_desktop_viewport = XInternAtom (dpy, "_NET_DESKTOP_VIEWPORT", FALSE);
+    net_desktop_names = XInternAtom (dpy, "_NET_DESKTOP_NAMES", FALSE);
     net_number_of_desktops =
         XInternAtom (dpy, "_NET_NUMBER_OF_DESKTOPS", FALSE);
+    net_showing_desktop = XInternAtom (dpy, "_NET_SHOWING_DESKTOP", FALSE);
     net_startup_id = XInternAtom (dpy, "_NET_STARTUP_ID", FALSE);
     net_supported = XInternAtom (dpy, "_NET_SUPPORTED", FALSE);
     net_supporting_wm_check =
@@ -444,6 +472,7 @@ initNetHints (Display * dpy)
     net_wm_state_sticky = XInternAtom (dpy, "_NET_WM_STATE_STICKY", FALSE);
     net_wm_state = XInternAtom (dpy, "_NET_WM_STATE", FALSE);
     net_wm_strut = XInternAtom (dpy, "_NET_WM_STRUT", FALSE);
+    net_wm_strut_partial = XInternAtom (dpy, "_NET_WM_STRUT_PARTIAL", FALSE);
     net_wm_window_type_desktop =
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_DESKTOP", FALSE);
     net_wm_window_type_dialog =
@@ -462,11 +491,12 @@ initNetHints (Display * dpy)
         XInternAtom (dpy, "_NET_WM_WINDOW_TYPE_UTILITY", FALSE);
     net_wm_window_type = XInternAtom (dpy, "_NET_WM_WINDOW_TYPE", FALSE);
     net_workarea = XInternAtom (dpy, "_NET_WORKAREA", FALSE);
+    net_wm_user_time = XInternAtom (dpy, "_NET_WM_USER_TIME", FALSE);
     utf8_string = XInternAtom (dpy, "UTF8_STRING", FALSE);
 }
 
 void
-set_net_supported_hint (Display * dpy, int screen, Window check_win)
+setNetSupportedHint (Display * dpy, int screen, Window check_win)
 {
     Atom atoms[64];
     unsigned long data[1];
@@ -479,7 +509,9 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
     atoms[i++] = net_current_desktop;
     atoms[i++] = net_desktop_geometry;
     atoms[i++] = net_desktop_viewport;
+    atoms[i++] = net_desktop_names;
     atoms[i++] = net_number_of_desktops;
+    atoms[i++] = net_showing_desktop;
     atoms[i++] = net_supported;
     atoms[i++] = net_supporting_wm_check;
     atoms[i++] = net_wm_action_change_desktop;
@@ -509,6 +541,7 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
     atoms[i++] = net_wm_state_skip_taskbar;
     atoms[i++] = net_wm_state_sticky;
     atoms[i++] = net_wm_strut;
+    atoms[i++] = net_wm_strut_partial;
     atoms[i++] = net_wm_window_type;
     atoms[i++] = net_wm_window_type_desktop;
     atoms[i++] = net_wm_window_type_dialog;
@@ -518,6 +551,7 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
     atoms[i++] = net_wm_window_type_splashscreen;
     atoms[i++] = net_wm_window_type_toolbar;
     atoms[i++] = net_wm_window_type_utility;
+    atoms[i++] = net_wm_user_time;
     atoms[i++] = net_workarea;
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 
@@ -534,7 +568,7 @@ set_net_supported_hint (Display * dpy, int screen, Window check_win)
 }
 
 gboolean
-get_atom_list (Display * dpy, Window w, Atom a, Atom ** atoms_p,
+getAtomList (Display * dpy, Window w, Atom a, Atom ** atoms_p,
     int *n_atoms_p)
 {
     Atom type;
@@ -571,7 +605,7 @@ get_atom_list (Display * dpy, Window w, Atom a, Atom ** atoms_p,
 }
 
 gboolean
-get_cardinal_list (Display * dpy, Window w, Atom xatom,
+getCardinalList (Display * dpy, Window w, Atom xatom,
     unsigned long **cardinals_p, int *n_cardinals_p)
 {
     Atom type;
@@ -604,25 +638,25 @@ get_cardinal_list (Display * dpy, Window w, Atom xatom,
 }
 
 void
-set_net_workarea (Display * dpy, int screen, int nb_workspaces, int * m)
+setNetWorkarea (Display * dpy, int screen, int nb_workspaces, int * m)
 {
     unsigned long *data, *ptr;
     int i, j;
 
-    TRACE ("entering set_net_workarea");
+    TRACE ("entering setNetWorkarea");
     j = (nb_workspaces ? nb_workspaces : 1);
     data = (unsigned long *) g_new0 (unsigned long, j * 4);
     ptr = data;
     for (i = 0; i < j; i++)
     {
-        *ptr++ = (unsigned long) m[MARGIN_LEFT];
-        *ptr++ = (unsigned long) m[MARGIN_TOP];
+        *ptr++ = (unsigned long) m[LEFT];
+        *ptr++ = (unsigned long) m[TOP];
         *ptr++ = (unsigned long) 
             (MyDisplayFullWidth (dpy, screen) - 
-                (m[MARGIN_LEFT] + m[MARGIN_RIGHT]));
+                (m[LEFT] + m[RIGHT]));
         *ptr++ = (unsigned long) 
             (MyDisplayFullHeight (dpy, screen) - 
-                (m[MARGIN_TOP] + m[MARGIN_BOTTOM]));
+                (m[TOP] + m[BOTTOM]));
     }
     XChangeProperty (dpy, RootWindow (dpy, screen), net_workarea, XA_CARDINAL,
         32, PropModeReplace, (unsigned char *) data, j * 4);
@@ -630,10 +664,10 @@ set_net_workarea (Display * dpy, int screen, int nb_workspaces, int * m)
 }
 
 void
-init_net_desktop_params (Display * dpy, int screen, int workspace)
+initNetDesktopParams (Display * dpy, int screen, int workspace)
 {
     unsigned long data[2];
-    TRACE ("entering init_net_desktop_params");
+    TRACE ("entering initNetDesktopParams");
     data[0] = MyDisplayFullWidth (dpy, screen);
     data[1] = MyDisplayFullHeight (dpy, screen);
     XChangeProperty (dpy, RootWindow (dpy, screen), net_desktop_geometry,
@@ -684,7 +718,7 @@ getTransientFor (Display * dpy, int screen, Window w, Window * transient_for)
 }
 
 gboolean
-get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
+getUTF8String (Display * dpy, Window w, Atom xatom, char **str_p, int *length)
 {
     Atom type;
     int format;
@@ -692,7 +726,7 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
     unsigned char *str;
     unsigned long n_items;
 
-    TRACE ("entering get_utf8_string");
+    TRACE ("entering getUTF8String");
 
     *str_p = NULL;
     if ((XGetWindowProperty (dpy, w, xatom, 0, G_MAXLONG, FALSE, utf8_string,
@@ -733,10 +767,11 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
 
     if (str)
     {
-        utf8_string_remove_controls((gchar *) str, -1, NULL);
+        xfce_utf8_remove_controls((gchar *) str, -1, NULL);
     }
     
     *str_p = str;
+    *length = n_items;
 
     return TRUE;
 }
@@ -779,7 +814,7 @@ get_text_property (Display * dpy, Window w, Atom a)
         retval = text_property_to_utf8 (dpy, &text);
         if (retval)
         {
-            utf8_string_remove_controls((gchar *) retval, -1, NULL);
+            xfce_utf8_remove_controls((gchar *) retval, -1, NULL);
         }
         if ((text.value) && (text.nitems > 0))
         {
@@ -799,6 +834,7 @@ void
 getWindowName (Display * dpy, Window w, char **name)
 {
     char *str;
+    int len;
 
     TRACE ("entering getWindowName");
 
@@ -806,7 +842,7 @@ getWindowName (Display * dpy, Window w, char **name)
     *name = NULL;
     g_return_if_fail (w != None);
 
-    if (get_utf8_string (dpy, w, net_wm_name, &str))
+    if (getUTF8String (dpy, w, net_wm_name, &str, &len))
     {
         *name = strdup (str);
         XFree (str);
@@ -900,15 +936,12 @@ getWindowRole (Display * dpy, Window window, char **role)
 
     if (XGetTextProperty (dpy, window, &tp, wm_window_role))
     {
-        if (tp.value)
+        if ((tp.value) && (tp.encoding == XA_STRING) && (tp.format == 8)
+            && (tp.nitems != 0))
         {
-            if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
-            {
-                *role = strdup ((char *) tp.value);
-                XFree (tp.value);
-                return TRUE;
-            }
+            *role = strdup ((char *) tp.value);
             XFree (tp.value);
+            return TRUE;
         }
     }
 
@@ -937,15 +970,41 @@ getClientLeader (Display * dpy, Window window)
             && (nitems == 1) && (bytes_after == 0))
         {
             client_leader = *((Window *) prop);
-        }
-        if (prop)
-        {
             XFree (prop);
         }
     }
     return client_leader;
 }
 
+gboolean
+getNetWMUserTime (Display * dpy, Window window, Time *time)
+{
+    Atom actual_type;
+    int actual_format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    unsigned char *data = NULL;
+
+    TRACE ("entering getNetWMUserTime");
+
+    g_return_val_if_fail (window != None, None);
+
+    if (XGetWindowProperty (dpy, window, net_wm_user_time, 0L, 1L, FALSE,
+            XA_CARDINAL, &actual_type, &actual_format, &nitems,
+            &bytes_after, &data) == Success)
+    {
+        if ((data) && (actual_type == XA_CARDINAL)
+            && (nitems == 1) && (bytes_after == 0))
+        {
+            *time = *((long *) data);
+            XFree (data);
+            return TRUE;
+        }
+    }
+    *time = 0L;
+    return FALSE;
+}
+
 gboolean
 getClientID (Display * dpy, Window window, char **client_id)
 {
@@ -1010,15 +1069,12 @@ getWindowStartupId (Display * dpy, Window window, char **startup_id)
 
     if (XGetTextProperty (dpy, window, &tp, net_startup_id))
     {
-        if (tp.value)
+        if ((tp.value) && (tp.encoding == XA_STRING) && (tp.format == 8)
+            && (tp.nitems != 0))
         {
-            if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
-            {
-                *startup_id = g_strdup ((char *) tp.value);
-                XFree (tp.value);
-                return TRUE;
-            }
+            *startup_id = strdup ((char *) tp.value);
             XFree (tp.value);
+            return TRUE;
         }
     }
 
