@@ -33,9 +33,9 @@ void getMouseXY (Window, int *, int *);
 Window getMouseWindow (Window);
 GC createGC (Colormap, char *, int, XFontStruct *, int, gboolean);
 void sendClientMessage (Window, Atom, Atom, Time);
-void MyXGrabServer (void);
-void MyXUngrabServer (void);
-gboolean MyCheckWindow(Window);
+void myXGrabServer (void);
+void myXUngrabServer (void);
+gboolean myCheckWindow(Window);
 Window setTmpEventWin (int, int, unsigned int, unsigned int, long);
 void removeTmpEventWin (Window);
 void placeSidewalks(gboolean);
