@@ -43,8 +43,13 @@
 
 #define CHANNEL1 "xfwm4"
 #define CHANNEL2 "margins"
+#define CHANNEL3 "workspaces"
 #define TOINT(x) (x ? atoi(x) : 0)
-#define DEFAULT_KEYTHEME "default.keys"
+#define DEFAULT_KEYTHEME "Default"
+#define KEYTHEMERC "keythemerc"
+#define THEMERC    "themerc"
+#define WS_SEP ';'
+#define WS_SEP_S ";"
 
 Params params;
 
@@ -58,8 +63,8 @@ set_settings_margin (int idx, int value)
 
     switch (idx)
     {
-        case MARGIN_LEFT:
-        case MARGIN_RIGHT:
+        case LEFT:
+        case RIGHT:
             if (value < 0)
             {
                 val = 0;
@@ -74,8 +79,8 @@ set_settings_margin (int idx, int value)
             }
             params.xfwm_margins[idx] = val;
             break;
-        case MARGIN_TOP:
-        case MARGIN_BOTTOM:
+        case TOP:
+        case BOTTOM:
             if (value < 0)
             {
                 val = 0;
@@ -215,22 +220,46 @@ notify_cb (const char *name, const char *channel_name, McsAction action,
                 {
                     if (!strcmp (name, "Xfwm/LeftMargin"))
                     {
-                        set_settings_margin (MARGIN_LEFT,
+                        set_settings_margin (LEFT,
                             setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/RightMargin"))
                     {
-                        set_settings_margin (MARGIN_RIGHT,
+                        set_settings_margin (RIGHT,
                             setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/BottomMargin"))
                     {
-                        set_settings_margin (MARGIN_BOTTOM,
+                        set_settings_margin (BOTTOM,
                             setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/TopMargin"))
                     {
-                        set_settings_margin (MARGIN_TOP, setting->data.v_int);
+                        set_settings_margin (TOP, setting->data.v_int);
+                    }
+                }
+                break;
+            case MCS_ACTION_DELETED:
+            default:
+                break;
+        }
+    }
+    else if (!g_ascii_strcasecmp (CHANNEL3, channel_name))
+    {
+        switch (action)
+        {
+            case MCS_ACTION_NEW:
+                /* The following is to reduce initial startup time and reloads */
+                if (!mcs_initted)
+                {
+                    return;
+                }
+            case MCS_ACTION_CHANGED:
+                if (setting->type == MCS_TYPE_INT)
+                {
+                    if (!strcmp (name, "Xfwm/WorkspaceCount"))
+                    {
+                        workspaceSetCount(setting->data.v_int);
                     }
                 }
                 break;
@@ -284,7 +313,7 @@ loadRcData (Settings rc[])
     gchar *keytheme;
     gchar *keythemevalue;
 
-    if (!parseRc ("defaults", DATADIR, rc))
+    if (!parseRc ("defaults", PACKAGE_DATADIR, rc))
     {
         g_warning (_("%s: Missing defaults file"), progname);
         exit (1);
@@ -292,8 +321,8 @@ loadRcData (Settings rc[])
     keythemevalue = getValue ("keytheme", rc);
     if (keythemevalue)
     {
-        keytheme = getThemeDir (keythemevalue);
-        parseRc ("keythemerc", keytheme, rc);
+        keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
+        parseRc (KEYTHEMERC, keytheme, rc);
         g_free (keytheme);
     }
     parseRc ("xfwm4rc", homedir, rc);
@@ -444,6 +473,12 @@ loadMcsData (Settings rc[])
             setIntValueFromInt ("margin_top", setting->data.v_int, rc);
             mcs_setting_free (setting);
         }
+        if (mcs_client_get_setting (client, "Xfwm/WorkspaceCount", CHANNEL3,
+                &setting) == MCS_SUCCESS)
+        {
+            setIntValueFromInt ("workspace_count", setting->data.v_int, rc);
+            mcs_setting_free (setting);
+        }
     }
 }
 
@@ -478,8 +513,8 @@ loadTheme (Settings rc[])
     rc[18].value = get_style (widget, "dark", "normal");
     rc[19].value = get_style (widget, "mid", "normal");
 
-    theme = getThemeDir (getValue ("theme", rc));
-    parseRc ("themerc", theme, rc);
+    theme = getThemeDir (getValue ("theme", rc), THEMERC);
+    parseRc (THEMERC, theme, rc);
 
     for (i = 0; i < 20; i++)
     {
@@ -582,112 +617,111 @@ loadTheme (Settings rc[])
         if (desc)
         {
             gtk_widget_modify_font (widget, desc);
-            pango_font_description_free (desc);
         }
     }
 
     myPixmapLoad (dpy, &params.sides[SIDE_LEFT][ACTIVE], theme,
-        "left-active.xpm", colsym, 20);
+        "left-active", colsym, 20);
     myPixmapLoad (dpy, &params.sides[SIDE_LEFT][INACTIVE], theme,
-        "left-inactive.xpm", colsym, 20);
+        "left-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.sides[SIDE_RIGHT][ACTIVE], theme,
-        "right-active.xpm", colsym, 20);
+        "right-active", colsym, 20);
     myPixmapLoad (dpy, &params.sides[SIDE_RIGHT][INACTIVE], theme,
-        "right-inactive.xpm", colsym, 20);
+        "right-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.sides[SIDE_BOTTOM][ACTIVE], theme,
-        "bottom-active.xpm", colsym, 20);
+        "bottom-active", colsym, 20);
     myPixmapLoad (dpy, &params.sides[SIDE_BOTTOM][INACTIVE], theme,
-        "bottom-inactive.xpm", colsym, 20);
+        "bottom-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_TOP_LEFT][ACTIVE], theme,
-        "top-left-active.xpm", colsym, 20);
+        "top-left-active", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_TOP_LEFT][INACTIVE], theme,
-        "top-left-inactive.xpm", colsym, 20);
+        "top-left-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_TOP_RIGHT][ACTIVE], theme,
-        "top-right-active.xpm", colsym, 20);
+        "top-right-active", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_TOP_RIGHT][INACTIVE], theme,
-        "top-right-inactive.xpm", colsym, 20);
+        "top-right-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_LEFT][ACTIVE], theme,
-        "bottom-left-active.xpm", colsym, 20);
+        "bottom-left-active", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_LEFT][INACTIVE], theme,
-        "bottom-left-inactive.xpm", colsym, 20);
+        "bottom-left-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_RIGHT][ACTIVE], theme,
-        "bottom-right-active.xpm", colsym, 20);
+        "bottom-right-active", colsym, 20);
     myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_RIGHT][INACTIVE], theme,
-        "bottom-right-inactive.xpm", colsym, 20);
+        "bottom-right-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[HIDE_BUTTON][ACTIVE], theme,
-        "hide-active.xpm", colsym, 20);
+        "hide-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[HIDE_BUTTON][INACTIVE], theme,
-        "hide-inactive.xpm", colsym, 20);
+        "hide-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[HIDE_BUTTON][PRESSED], theme,
-        "hide-pressed.xpm", colsym, 20);
+        "hide-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[CLOSE_BUTTON][ACTIVE], theme,
-        "close-active.xpm", colsym, 20);
+        "close-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[CLOSE_BUTTON][INACTIVE], theme,
-        "close-inactive.xpm", colsym, 20);
+        "close-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[CLOSE_BUTTON][PRESSED], theme,
-        "close-pressed.xpm", colsym, 20);
+        "close-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][ACTIVE], theme,
-        "maximize-active.xpm", colsym, 20);
+        "maximize-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][INACTIVE], theme,
-        "maximize-inactive.xpm", colsym, 20);
+        "maximize-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][PRESSED], theme,
-        "maximize-pressed.xpm", colsym, 20);
+        "maximize-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][ACTIVE], theme,
-        "shade-active.xpm", colsym, 20);
+        "shade-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][INACTIVE], theme,
-        "shade-inactive.xpm", colsym, 20);
+        "shade-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][PRESSED], theme,
-        "shade-pressed.xpm", colsym, 20);
+        "shade-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][ACTIVE], theme,
-        "stick-active.xpm", colsym, 20);
+        "stick-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][INACTIVE], theme,
-        "stick-inactive.xpm", colsym, 20);
+        "stick-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][PRESSED], theme,
-        "stick-pressed.xpm", colsym, 20);
+        "stick-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MENU_BUTTON][ACTIVE], theme,
-        "menu-active.xpm", colsym, 20);
+        "menu-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MENU_BUTTON][INACTIVE], theme,
-        "menu-inactive.xpm", colsym, 20);
+        "menu-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MENU_BUTTON][PRESSED], theme,
-        "menu-pressed.xpm", colsym, 20);
+        "menu-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][T_ACTIVE], theme,
-        "shade-toggled-active.xpm", colsym, 20);
+        "shade-toggled-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][T_INACTIVE], theme,
-        "shade-toggled-inactive.xpm", colsym, 20);
+        "shade-toggled-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][T_PRESSED], theme,
-        "shade-toggled-pressed.xpm", colsym, 20);
+        "shade-toggled-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][T_ACTIVE], theme,
-        "stick-toggled-active.xpm", colsym, 20);
+        "stick-toggled-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][T_INACTIVE], theme,
-        "stick-toggled-inactive.xpm", colsym, 20);
+        "stick-toggled-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][T_PRESSED], theme,
-        "stick-toggled-pressed.xpm", colsym, 20);
+        "stick-toggled-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][T_ACTIVE], theme,
-        "maximize-toggled-active.xpm", colsym, 20);
+        "maximize-toggled-active", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][T_INACTIVE], theme,
-        "maximize-toggled-inactive.xpm", colsym, 20);
+        "maximize-toggled-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][T_PRESSED], theme,
-        "maximize-toggled-pressed.xpm", colsym, 20);
+        "maximize-toggled-pressed", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_1][ACTIVE], theme,
-        "title-1-active.xpm", colsym, 20);
+        "title-1-active", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_1][INACTIVE], theme,
-        "title-1-inactive.xpm", colsym, 20);
+        "title-1-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_2][ACTIVE], theme,
-        "title-2-active.xpm", colsym, 20);
+        "title-2-active", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_2][INACTIVE], theme,
-        "title-2-inactive.xpm", colsym, 20);
+        "title-2-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_3][ACTIVE], theme,
-        "title-3-active.xpm", colsym, 20);
+        "title-3-active", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_3][INACTIVE], theme,
-        "title-3-inactive.xpm", colsym, 20);
+        "title-3-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_4][ACTIVE], theme,
-        "title-4-active.xpm", colsym, 20);
+        "title-4-active", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_4][INACTIVE], theme,
-        "title-4-inactive.xpm", colsym, 20);
+        "title-4-inactive", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_5][ACTIVE], theme,
-        "title-5-active.xpm", colsym, 20);
+        "title-5-active", colsym, 20);
     myPixmapLoad (dpy, &params.title[TITLE_5][INACTIVE], theme,
-        "title-5-inactive.xpm", colsym, 20);
+        "title-5-inactive", colsym, 20);
 
     if (!g_ascii_strcasecmp ("left", getValue ("title_alignment", rc)))
     {
@@ -756,14 +790,14 @@ loadKeyBindings (Settings rc[])
     keythemevalue = getValue ("keytheme", rc);
     if (keythemevalue)
     {
-        keytheme = getThemeDir (keythemevalue);
-        if (!parseRc ("keythemerc", keytheme, rc))
+        keytheme = getThemeDir (keythemevalue, KEYTHEMERC);
+        if (!parseRc (KEYTHEMERC, keytheme, rc))
         {
             g_warning (_("%s: specified key theme \"%s\" missing, using default"),
                        progname, keythemevalue);
             g_free (keytheme);
-            keytheme = getThemeDir (DEFAULT_KEYTHEME);
-            parseRc ("keythemerc", keytheme, rc);
+            keytheme = getThemeDir (DEFAULT_KEYTHEME, KEYTHEMERC);
+            parseRc (KEYTHEMERC, keytheme, rc);
         }
         g_free (keytheme);
 
@@ -977,7 +1011,6 @@ loadSettings (void)
         {"cycle_windows_key", NULL, TRUE},
         {"del_workspace_key", NULL, TRUE},
         {"hide_window_key", NULL, TRUE},
-        {"lower_window_key", NULL, TRUE},
         {"maximize_horiz_key", NULL, TRUE},
         {"maximize_vert_key", NULL, TRUE},
         {"maximize_window_key", NULL, TRUE},
@@ -998,7 +1031,6 @@ loadSettings (void)
         {"move_window_workspace_9_key", NULL, TRUE},
         {"next_workspace_key", NULL, TRUE},
         {"prev_workspace_key", NULL, TRUE},
-        {"raise_window_key", NULL, TRUE},
         {"resize_window_down_key", NULL, TRUE},
         {"resize_window_left_key", NULL, TRUE},
         {"resize_window_right_key", NULL, TRUE},
@@ -1034,6 +1066,8 @@ loadSettings (void)
         {"shortcut_8_exec", NULL, FALSE},
         {"shortcut_9_exec", NULL, FALSE},
         {"shortcut_10_exec", NULL, FALSE},
+        {"raise_window_key", NULL, TRUE},
+        {"lower_window_key", NULL, TRUE},
         {NULL, NULL, FALSE}
     };
     GValue tmp_val = { 0, };
@@ -1077,11 +1111,11 @@ loadSettings (void)
     params.snap_width = abs (TOINT (getValue ("snap_width", rc)));
     params.dbl_click_time = abs (TOINT (getValue ("dbl_click_time", rc)));
 
-    set_settings_margin (MARGIN_LEFT, TOINT (getValue ("margin_left", rc)));
-    set_settings_margin (MARGIN_RIGHT, TOINT (getValue ("margin_right", rc)));
-    set_settings_margin (MARGIN_BOTTOM, TOINT (getValue ("margin_bottom",
+    set_settings_margin (LEFT, TOINT (getValue ("margin_left", rc)));
+    set_settings_margin (RIGHT, TOINT (getValue ("margin_right", rc)));
+    set_settings_margin (BOTTOM, TOINT (getValue ("margin_bottom",
                 rc)));
-    set_settings_margin (MARGIN_TOP, TOINT (getValue ("margin_top", rc)));
+    set_settings_margin (TOP, TOINT (getValue ("margin_top", rc)));
 
     g_value_init (&tmp_val, G_TYPE_INT);
     if (gdk_setting_get ("gtk-double-click-time", &tmp_val))
@@ -1191,6 +1225,7 @@ initSettings (void)
 {
     int i;
     long val = 0;
+    char *names = NULL;
 
     TRACE ("entering initSettings");
 
@@ -1241,6 +1276,7 @@ initSettings (void)
     {
         mcs_client_add_channel (client, CHANNEL1);
         mcs_client_add_channel (client, CHANNEL2);
+        mcs_client_add_channel (client, CHANNEL3);
     }
     else
     {
@@ -1258,6 +1294,16 @@ initSettings (void)
         workspaceSetCount (val);
     }
 
+    if (getUTF8String (dpy, root, net_desktop_names, &names, &i))
+    {
+        workspaceSetNames (names, i);
+    }
+    else
+    {
+        params.workspace_names = NULL;
+        params.workspace_names_length = 0;
+    }
+
     if (!loadSettings ())
     {
         return FALSE;
